@@ -68,6 +68,14 @@ def list_test_queues(format:str=None):
 			, 'sub_analysis': 'windowed'
 			, 'datum': 'dehli_climate.parquet'	
 		},
+		{
+			'queue_name': 'keras_image_forecast'
+			, 'data_type': 'image'
+			, 'supervision': 'unsupervised'
+			, 'analysis': 'regression'
+			, 'sub_analysis': 'windowed'
+			, 'datum': 'liberty_moon.csv'
+		},
 		{
 			'queue_name': 'pytorch_multiclass'
 			, 'data_type': 'tabular'
@@ -547,7 +555,15 @@ def make_test_queue_keras_regression(repeat_count:int=1, fold_count:int=None):
 # ------------------------ KERAS IMAGE BINARY ------------------------
 def keras_image_binary_fn_build(features_shape, label_shape, **hp):
 	model = keras.models.Sequential()
-	
+	# incoming features_shape = channels * rows * columns
+	# https://keras.io/api/layers/reshaping_layers/reshape/
+	# https://www.tensorflow.org/api_docs/python/tf/keras/layers/Conv1D
+	# Conv1D shape = `batch_shape + (steps, input_dim)`
+	model.add(layers.Reshape(
+		(features_shape[1],features_shape[2])#,features_shape[0])#dropping
+		, input_shape=features_shape)
+	)
+
 	model.add(layers.Conv1D(128*hp['neuron_multiply'], kernel_size=hp['kernel_size'], input_shape=features_shape, padding='same', activation='relu', kernel_initializer=hp['cnn_init']))
 	model.add(layers.MaxPooling1D(pool_size=hp['pool_size']))
 	model.add(layers.Dropout(hp['dropout']))
@@ -617,7 +633,7 @@ def make_test_queue_keras_image_binary(repeat_count:int=1, fold_count:int=None):
 
 	# Dataset.Image
 	image_urls = datum.get_remote_urls(manifest_name='brain_tumor.csv')
-	dataset_image = Dataset.Image.from_urls(urls = image_urls)
+	dataset_image = Dataset.Image.from_urls_pillow(urls=image_urls)
 	feature = dataset_image.make_feature()
 	
 	if (fold_count is not None):
@@ -1347,6 +1363,9 @@ def pytorch_image_binary_fn_build(features_shape, label_shape, **hp):
 	return model
 
 def pytorch_image_binary_fn_train(model, loser, optimizer, samples_train, samples_evaluate, **hp):   
+	# incoming features_shape = channels * rows * columns
+	#https://pytorch.org/docs/stable/generated/torch.nn.Conv1d.html#torch.nn.Conv1d
+	#https://pytorch.org/docs/stable/generated/torch.reshape.html
 	## --- Prepare mini batches for analysis ---
 	batched_features, batched_labels = aiqc.torch_batcher(
 		samples_train['features'], samples_train['labels'],
@@ -1389,17 +1408,25 @@ def pytorch_image_binary_fn_train(model, loser, optimizer, samples_train, sample
 		history['val_accuracy'].append(float(eval_acc))
 	return model, history
 
+def pytorch_image_binary_fn_predict(model, samples_predict):
+	probability = model(samples_predict['features'])
+	# Convert tensor back to numpy for AIQC metrics.
+	probability = probability.detach().numpy()
+	prediction = (probability > 0.5).astype("int32")
+	# Both objects are numpy.
+	return prediction, probability
+
 def make_test_queue_pytorch_image_binary(repeat_count:int=1, fold_count:int=None):
 	df = datum.to_pandas(name='brain_tumor.csv')
-
 	# Dataset.Tabular
 	dataset_tabular = Dataset.Tabular.from_pandas(dataframe=df)
 	label = dataset_tabular.make_label(columns=['status'])
 
 	# Dataset.Image
 	image_urls = datum.get_remote_urls(manifest_name='brain_tumor.csv')
-	dataset_image = Dataset.Image.from_urls(urls = image_urls)
+	dataset_image = Dataset.Image.from_urls_pillow(urls=image_urls)
 	feature = dataset_image.make_feature()
+	feature.make_featureshaper(reshape_indices=(0,2,3))
 	
 	if (fold_count is not None):
 		size_test = 0.25
@@ -1428,6 +1455,7 @@ def make_test_queue_pytorch_image_binary(repeat_count:int=1, fold_count:int=None
 		, analysis_type = "classification_binary"
 		, fn_build = pytorch_image_binary_fn_build
 		, fn_train = pytorch_image_binary_fn_train
+		, fn_predict = pytorch_image_binary_fn_predict
 	)
 
 	queue = algorithm.make_queue(
@@ -1438,6 +1466,125 @@ def make_test_queue_pytorch_image_binary(repeat_count:int=1, fold_count:int=None
 	)
 	return queue
 
+
+# ------------------------ KERAS IMAGE FORECAST ------------------------
+def keras_image_forecast_fn_build(features_shape, label_shape, **hp):
+	"""
+	- Model size was too large for SQLite blob... need to rework to store it on the FS.
+	- incoming features_shape = frame* channels * rows * columns
+	- https://keras.io/api/layers/reshaping_layers/reshape/
+	- Model size was too large for SQLite blob... need to rework to store it on the FS.
+	- ConvLSTM1D is still in nightly build so use ConvLSTM2D
+	- https://www.tensorflow.org/api_docs/python/tf/keras/layers/ConvLSTM2D
+	- If data_format='channels_last' 5D tensor with shape: (samples, time, rows, cols, channels)
+	"""
+	model = keras.models.Sequential()
+	model.add(layers.Conv1D(64*hp['multiplier'], 3, activation=hp['activation'], padding='same'))
+	model.add(layers.MaxPool1D( 2, padding='same'))
+	model.add(layers.Conv1D(32*hp['multiplier'], 3, activation=hp['activation'], padding='same'))
+	model.add(layers.MaxPool1D( 2, padding='same'))
+	model.add(layers.Conv1D(16*hp['multiplier'], 3, activation=hp['activation'], padding='same'))
+	model.add(layers.MaxPool1D( 2, padding='same'))
+
+	# decoding architecture
+	model.add(layers.Conv1D(16*hp['multiplier'], 3, activation=hp['activation'], padding='same'))
+	model.add(layers.UpSampling1D(2))
+	model.add(layers.Conv1D(32*hp['multiplier'], 3, activation=hp['activation'], padding='same'))
+	model.add(layers.UpSampling1D(2))
+	model.add(layers.Conv1D(64*hp['multiplier'], 3, activation=hp['activation']))
+	model.add(layers.UpSampling1D(2))
+	model.add(layers.Conv1D(50, 3, activation='relu', padding='same'))# removing sigmoid
+	return model
+
+def keras_image_forecast_fn_train(model, loser, optimizer, samples_train, samples_evaluate, **hp):
+	model.compile(
+		optimizer=optimizer
+		, loss=loser
+		, metrics=['mean_squared_error']
+	)
+	
+	model.fit(
+		samples_train["features"]
+		, samples_train["labels"]
+		, validation_data = (
+			samples_evaluate["features"]
+			, samples_evaluate["labels"]
+		)
+		, verbose = 0
+		, batch_size = hp['batch_size']
+		, callbacks=[keras.callbacks.History()]
+		, epochs = hp['epoch_count']
+	)
+	return model
+
+# def keras_image_forecast_fn_lose(**hp):
+# 	loser = keras.losses.BCEWithLogitsLoss()
+# 	return loser
+
+def make_test_queue_keras_image_forecast(repeat_count:int=1, fold_count:int=None):
+	folder_path = 'remote_datum/image/liberty_moon/images'
+	image_dataset = Dataset.Image.from_folder_pillow(folder_path=folder_path, ingest=False, dtype='float64')
+
+	feature = image_dataset.make_feature()
+	feature.make_window(size_window=1, size_shift=2)
+	encoderset = feature.make_encoderset()
+	encoderset.make_featurecoder(
+		sklearn_preprocess= FunctionTransformer(aiqc.div255, inverse_func=aiqc.mult255)
+		, dtypes = 'float64'
+	)
+	feature.make_featureshaper(reshape_indices=(0,3,4))
+
+	if (fold_count is not None):
+		size_test = 0.15
+		size_validation = None
+	elif (fold_count is None):
+		size_test = 0.15
+		size_validation = None#small dataset
+
+	splitset = Splitset.make(
+		feature_ids = feature.id
+		, size_test = size_test
+		, size_validation = size_validation
+	)
+
+	if (fold_count is not None):
+		foldset = splitset.make_foldset(
+			fold_count = fold_count
+		)
+		foldset_id = foldset.id
+	else:
+		foldset_id = None
+
+	algorithm = Algorithm.make(
+		library = "keras"
+		, analysis_type = "regression"
+		, fn_build = keras_image_forecast_fn_build
+		, fn_train = keras_image_forecast_fn_train
+		# , fn_lose = keras_image_forecast_fn_lose
+	)
+
+	hyperparameters = dict(
+		epoch_count = [150]
+		, batch_size = [1]
+		, cnn_init = ['he_normal']
+		, activation = ['relu']
+		, multiplier = [3]
+	)
+
+	hyperparamset = algorithm.make_hyperparamset(
+		hyperparameters = hyperparameters
+	)
+
+	queue = algorithm.make_queue(
+		splitset_id = splitset.id
+		, foldset_id = foldset_id
+		, hyperparamset_id = hyperparamset.id
+		, repeat_count = repeat_count
+	)
+	return queue
+
+	
+
 # ------------------------ TEST BATCH CALLER ------------------------
 def make_test_queue(name:str, repeat_count:int=1, fold_count:int=None):
 	if (name == 'keras_multiclass'):
@@ -1454,6 +1601,8 @@ def make_test_queue(name:str, repeat_count:int=1, fold_count:int=None):
 		queue = make_test_queue_keras_sequence_binary(repeat_count, fold_count)
 	elif (name == 'keras_tabular_forecast'):
 		queue = make_test_queue_keras_tabular_forecast(repeat_count, fold_count)
+	elif (name == 'keras_image_forecast'):
+		queue = make_test_queue_keras_image_forecast(repeat_count, fold_count)
 	elif (name == 'pytorch_binary'):
 		queue = make_test_queue_pytorch_binary(repeat_count, fold_count)
 	elif (name == 'pytorch_multiclass'):
